# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator 0.17.0.0
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::ARM::Dns
  #
  # RecordSets
  #
  class RecordSets
    include Azure::ARM::Dns::Models
    include MsRestAzure

    #
    # Creates and initializes a new instance of the RecordSets class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [DnsManagementClient] reference to the DnsManagementClient
    attr_reader :client

    #
    # Updates a RecordSet within a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param zone_name [String] The name of the zone without a terminating dot.
    # @param relative_record_set_name [String] The name of the RecordSet, relative
    # to the name of the zone.
    # @param record_type [RecordType] The type of DNS record. Possible values
    # include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'
    # @param parameters [RecordSet] Parameters supplied to the Update operation.
    # @param if_match [String] The etag of Zone.
    # @param if_none_match [String] Defines the If-None-Match condition. Set to
    # '*' to force Create-If-Not-Exist. Other values will be ignored.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [RecordSet] operation results.
    #
    def update(resource_group_name, zone_name, relative_record_set_name, record_type, parameters, if_match = nil, if_none_match = nil, custom_headers = nil)
      response = update_async(resource_group_name, zone_name, relative_record_set_name, record_type, parameters, if_match, if_none_match, custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Updates a RecordSet within a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param zone_name [String] The name of the zone without a terminating dot.
    # @param relative_record_set_name [String] The name of the RecordSet, relative
    # to the name of the zone.
    # @param record_type [RecordType] The type of DNS record. Possible values
    # include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'
    # @param parameters [RecordSet] Parameters supplied to the Update operation.
    # @param if_match [String] The etag of Zone.
    # @param if_none_match [String] Defines the If-None-Match condition. Set to
    # '*' to force Create-If-Not-Exist. Other values will be ignored.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def update_with_http_info(resource_group_name, zone_name, relative_record_set_name, record_type, parameters, if_match = nil, if_none_match = nil, custom_headers = nil)
      update_async(resource_group_name, zone_name, relative_record_set_name, record_type, parameters, if_match, if_none_match, custom_headers).value!
    end

    #
    # Updates a RecordSet within a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param zone_name [String] The name of the zone without a terminating dot.
    # @param relative_record_set_name [String] The name of the RecordSet, relative
    # to the name of the zone.
    # @param record_type [RecordType] The type of DNS record. Possible values
    # include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'
    # @param parameters [RecordSet] Parameters supplied to the Update operation.
    # @param if_match [String] The etag of Zone.
    # @param if_none_match [String] Defines the If-None-Match condition. Set to
    # '*' to force Create-If-Not-Exist. Other values will be ignored.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def update_async(resource_group_name, zone_name, relative_record_set_name, record_type, parameters, if_match = nil, if_none_match = nil, custom_headers = nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'zone_name is nil' if zone_name.nil?
      fail ArgumentError, 'relative_record_set_name is nil' if relative_record_set_name.nil?
      fail ArgumentError, 'record_type is nil' if record_type.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['If-Match'] = if_match unless if_match.nil?
      request_headers['If-None-Match'] = if_none_match unless if_none_match.nil?
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Serialize Request
      request_mapper = RecordSet.mapper()
      request_content = @client.serialize(request_mapper,  parameters, 'parameters')
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/dnszones/{zoneName}/{recordType}/{relativeRecordSetName}'
      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'zoneName' => zone_name,'recordType' => record_type,'subscriptionId' => @client.subscription_id},
          skip_encoding_path_params: {'relativeRecordSetName' => relative_record_set_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {})
      }

      request_url = @base_url || @client.base_url

      request = MsRest::HttpOperationRequest.new(request_url, path_template, :patch, options)
      promise = request.run_promise do |req|
        @client.credentials.sign_request(req) unless @client.credentials.nil?
      end

      promise = promise.then do |http_response|
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(request, http_response, error_model)
        end

        # Create Result
        result = MsRestAzure::AzureOperationResponse.new(request, http_response)
        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = RecordSet.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response, 'result.body')
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates or Updates a RecordSet within a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param zone_name [String] The name of the zone without a terminating dot.
    # @param relative_record_set_name [String] The name of the RecordSet, relative
    # to the name of the zone.
    # @param record_type [RecordType] The type of DNS record. Possible values
    # include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'
    # @param parameters [RecordSet] Parameters supplied to the CreateOrUpdate
    # operation.
    # @param if_match [String] The etag of Recordset.
    # @param if_none_match [String] Defines the If-None-Match condition. Set to
    # '*' to force Create-If-Not-Exist. Other values will be ignored.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [RecordSet] operation results.
    #
    def create_or_update(resource_group_name, zone_name, relative_record_set_name, record_type, parameters, if_match = nil, if_none_match = nil, custom_headers = nil)
      response = create_or_update_async(resource_group_name, zone_name, relative_record_set_name, record_type, parameters, if_match, if_none_match, custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates or Updates a RecordSet within a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param zone_name [String] The name of the zone without a terminating dot.
    # @param relative_record_set_name [String] The name of the RecordSet, relative
    # to the name of the zone.
    # @param record_type [RecordType] The type of DNS record. Possible values
    # include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'
    # @param parameters [RecordSet] Parameters supplied to the CreateOrUpdate
    # operation.
    # @param if_match [String] The etag of Recordset.
    # @param if_none_match [String] Defines the If-None-Match condition. Set to
    # '*' to force Create-If-Not-Exist. Other values will be ignored.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_or_update_with_http_info(resource_group_name, zone_name, relative_record_set_name, record_type, parameters, if_match = nil, if_none_match = nil, custom_headers = nil)
      create_or_update_async(resource_group_name, zone_name, relative_record_set_name, record_type, parameters, if_match, if_none_match, custom_headers).value!
    end

    #
    # Creates or Updates a RecordSet within a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param zone_name [String] The name of the zone without a terminating dot.
    # @param relative_record_set_name [String] The name of the RecordSet, relative
    # to the name of the zone.
    # @param record_type [RecordType] The type of DNS record. Possible values
    # include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'
    # @param parameters [RecordSet] Parameters supplied to the CreateOrUpdate
    # operation.
    # @param if_match [String] The etag of Recordset.
    # @param if_none_match [String] Defines the If-None-Match condition. Set to
    # '*' to force Create-If-Not-Exist. Other values will be ignored.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_or_update_async(resource_group_name, zone_name, relative_record_set_name, record_type, parameters, if_match = nil, if_none_match = nil, custom_headers = nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'zone_name is nil' if zone_name.nil?
      fail ArgumentError, 'relative_record_set_name is nil' if relative_record_set_name.nil?
      fail ArgumentError, 'record_type is nil' if record_type.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['If-Match'] = if_match unless if_match.nil?
      request_headers['If-None-Match'] = if_none_match unless if_none_match.nil?
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Serialize Request
      request_mapper = RecordSet.mapper()
      request_content = @client.serialize(request_mapper,  parameters, 'parameters')
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/dnszones/{zoneName}/{recordType}/{relativeRecordSetName}'
      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'zoneName' => zone_name,'recordType' => record_type,'subscriptionId' => @client.subscription_id},
          skip_encoding_path_params: {'relativeRecordSetName' => relative_record_set_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {})
      }

      request_url = @base_url || @client.base_url

      request = MsRest::HttpOperationRequest.new(request_url, path_template, :put, options)
      promise = request.run_promise do |req|
        @client.credentials.sign_request(req) unless @client.credentials.nil?
      end

      promise = promise.then do |http_response|
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 201 || status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(request, http_response, error_model)
        end

        # Create Result
        result = MsRestAzure::AzureOperationResponse.new(request, http_response)
        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 201
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = RecordSet.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response, 'result.body')
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = RecordSet.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response, 'result.body')
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Removes a RecordSet from a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param zone_name [String] The name of the zone without a terminating dot.
    # @param relative_record_set_name [String] The name of the RecordSet, relative
    # to the name of the zone.
    # @param record_type [RecordType] The type of DNS record. Possible values
    # include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'
    # @param if_match [String] Defines the If-Match condition. The delete
    # operation will be performed only if the ETag of the zone on the server
    # matches this value.
    # @param if_none_match [String] Defines the If-None-Match condition. The
    # delete operation will be performed only if the ETag of the zone on the
    # server does not match this value.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    #
    def delete(resource_group_name, zone_name, relative_record_set_name, record_type, if_match = nil, if_none_match = nil, custom_headers = nil)
      response = delete_async(resource_group_name, zone_name, relative_record_set_name, record_type, if_match, if_none_match, custom_headers).value!
      nil
    end

    #
    # Removes a RecordSet from a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param zone_name [String] The name of the zone without a terminating dot.
    # @param relative_record_set_name [String] The name of the RecordSet, relative
    # to the name of the zone.
    # @param record_type [RecordType] The type of DNS record. Possible values
    # include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'
    # @param if_match [String] Defines the If-Match condition. The delete
    # operation will be performed only if the ETag of the zone on the server
    # matches this value.
    # @param if_none_match [String] Defines the If-None-Match condition. The
    # delete operation will be performed only if the ETag of the zone on the
    # server does not match this value.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def delete_with_http_info(resource_group_name, zone_name, relative_record_set_name, record_type, if_match = nil, if_none_match = nil, custom_headers = nil)
      delete_async(resource_group_name, zone_name, relative_record_set_name, record_type, if_match, if_none_match, custom_headers).value!
    end

    #
    # Removes a RecordSet from a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param zone_name [String] The name of the zone without a terminating dot.
    # @param relative_record_set_name [String] The name of the RecordSet, relative
    # to the name of the zone.
    # @param record_type [RecordType] The type of DNS record. Possible values
    # include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'
    # @param if_match [String] Defines the If-Match condition. The delete
    # operation will be performed only if the ETag of the zone on the server
    # matches this value.
    # @param if_none_match [String] Defines the If-None-Match condition. The
    # delete operation will be performed only if the ETag of the zone on the
    # server does not match this value.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def delete_async(resource_group_name, zone_name, relative_record_set_name, record_type, if_match = nil, if_none_match = nil, custom_headers = nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'zone_name is nil' if zone_name.nil?
      fail ArgumentError, 'relative_record_set_name is nil' if relative_record_set_name.nil?
      fail ArgumentError, 'record_type is nil' if record_type.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['If-Match'] = if_match unless if_match.nil?
      request_headers['If-None-Match'] = if_none_match unless if_none_match.nil?
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/dnszones/{zoneName}/{recordType}/{relativeRecordSetName}'
      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'zoneName' => zone_name,'recordType' => record_type,'subscriptionId' => @client.subscription_id},
          skip_encoding_path_params: {'relativeRecordSetName' => relative_record_set_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {})
      }

      request_url = @base_url || @client.base_url

      request = MsRest::HttpOperationRequest.new(request_url, path_template, :delete, options)
      promise = request.run_promise do |req|
        @client.credentials.sign_request(req) unless @client.credentials.nil?
      end

      promise = promise.then do |http_response|
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 204 || status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(request, http_response, error_model)
        end

        # Create Result
        result = MsRestAzure::AzureOperationResponse.new(request, http_response)
        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?

        result
      end

      promise.execute
    end

    #
    # Gets a RecordSet.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param zone_name [String] The name of the zone without a terminating dot.
    # @param relative_record_set_name [String] The name of the RecordSet, relative
    # to the name of the zone.
    # @param record_type [RecordType] The type of DNS record. Possible values
    # include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [RecordSet] operation results.
    #
    def get(resource_group_name, zone_name, relative_record_set_name, record_type, custom_headers = nil)
      response = get_async(resource_group_name, zone_name, relative_record_set_name, record_type, custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets a RecordSet.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param zone_name [String] The name of the zone without a terminating dot.
    # @param relative_record_set_name [String] The name of the RecordSet, relative
    # to the name of the zone.
    # @param record_type [RecordType] The type of DNS record. Possible values
    # include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(resource_group_name, zone_name, relative_record_set_name, record_type, custom_headers = nil)
      get_async(resource_group_name, zone_name, relative_record_set_name, record_type, custom_headers).value!
    end

    #
    # Gets a RecordSet.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param zone_name [String] The name of the zone without a terminating dot.
    # @param relative_record_set_name [String] The name of the RecordSet, relative
    # to the name of the zone.
    # @param record_type [RecordType] The type of DNS record. Possible values
    # include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA', 'SRV', 'TXT'
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(resource_group_name, zone_name, relative_record_set_name, record_type, custom_headers = nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'zone_name is nil' if zone_name.nil?
      fail ArgumentError, 'relative_record_set_name is nil' if relative_record_set_name.nil?
      fail ArgumentError, 'record_type is nil' if record_type.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/dnszones/{zoneName}/{recordType}/{relativeRecordSetName}'
      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'zoneName' => zone_name,'recordType' => record_type,'subscriptionId' => @client.subscription_id},
          skip_encoding_path_params: {'relativeRecordSetName' => relative_record_set_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {})
      }

      request_url = @base_url || @client.base_url

      request = MsRest::HttpOperationRequest.new(request_url, path_template, :get, options)
      promise = request.run_promise do |req|
        @client.credentials.sign_request(req) unless @client.credentials.nil?
      end

      promise = promise.then do |http_response|
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(request, http_response, error_model)
        end

        # Create Result
        result = MsRestAzure::AzureOperationResponse.new(request, http_response)
        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = RecordSet.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response, 'result.body')
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists the RecordSets of a specified type in a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the zone.
    # @param zone_name [String] The name of the zone from which to enumerate
    # RecordsSets.
    # @param record_type [RecordType] The type of record sets to enumerate.
    # Possible values include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA',
    # 'SRV', 'TXT'
    # @param top [String] Query parameters. If null is passed returns the default
    # number of zones.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [RecordSetListResult] which provide lazy access to pages of the
    # response.
    #
    def list_by_type_as_lazy(resource_group_name, zone_name, record_type, top = nil, custom_headers = nil)
      response = list_by_type_async(resource_group_name, zone_name, record_type, top, custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_link|
          list_by_type_next_async(next_link, custom_headers)
        end
        page
      end
    end

    #
    # Lists the RecordSets of a specified type in a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the zone.
    # @param zone_name [String] The name of the zone from which to enumerate
    # RecordsSets.
    # @param record_type [RecordType] The type of record sets to enumerate.
    # Possible values include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA',
    # 'SRV', 'TXT'
    # @param top [String] Query parameters. If null is passed returns the default
    # number of zones.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<RecordSet>] operation results.
    #
    def list_by_type(resource_group_name, zone_name, record_type, top = nil, custom_headers = nil)
      first_page = list_by_type_as_lazy(resource_group_name, zone_name, record_type, top, custom_headers)
      first_page.get_all_items
    end

    #
    # Lists the RecordSets of a specified type in a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the zone.
    # @param zone_name [String] The name of the zone from which to enumerate
    # RecordsSets.
    # @param record_type [RecordType] The type of record sets to enumerate.
    # Possible values include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA',
    # 'SRV', 'TXT'
    # @param top [String] Query parameters. If null is passed returns the default
    # number of zones.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_type_with_http_info(resource_group_name, zone_name, record_type, top = nil, custom_headers = nil)
      list_by_type_async(resource_group_name, zone_name, record_type, top, custom_headers).value!
    end

    #
    # Lists the RecordSets of a specified type in a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the zone.
    # @param zone_name [String] The name of the zone from which to enumerate
    # RecordsSets.
    # @param record_type [RecordType] The type of record sets to enumerate.
    # Possible values include: 'A', 'AAAA', 'CNAME', 'MX', 'NS', 'PTR', 'SOA',
    # 'SRV', 'TXT'
    # @param top [String] Query parameters. If null is passed returns the default
    # number of zones.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_type_async(resource_group_name, zone_name, record_type, top = nil, custom_headers = nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'zone_name is nil' if zone_name.nil?
      fail ArgumentError, 'record_type is nil' if record_type.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/dnszones/{zoneName}/{recordType}'
      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'zoneName' => zone_name,'recordType' => record_type,'subscriptionId' => @client.subscription_id},
          query_params: {'$top' => top,'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {})
      }

      request_url = @base_url || @client.base_url

      request = MsRest::HttpOperationRequest.new(request_url, path_template, :get, options)
      promise = request.run_promise do |req|
        @client.credentials.sign_request(req) unless @client.credentials.nil?
      end

      promise = promise.then do |http_response|
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(request, http_response, error_model)
        end

        # Create Result
        result = MsRestAzure::AzureOperationResponse.new(request, http_response)
        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = RecordSetListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response, 'result.body')
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists all RecordSets in a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the zone.
    # @param zone_name [String] The name of the zone from which to enumerate
    # RecordSets.
    # @param top [String] Query parameters. If null is passed returns the default
    # number of zones.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [RecordSetListResult] which provide lazy access to pages of the
    # response.
    #
    def list_all_in_resource_group_as_lazy(resource_group_name, zone_name, top = nil, custom_headers = nil)
      response = list_all_in_resource_group_async(resource_group_name, zone_name, top, custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_link|
          list_all_in_resource_group_next_async(next_link, custom_headers)
        end
        page
      end
    end

    #
    # Lists all RecordSets in a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the zone.
    # @param zone_name [String] The name of the zone from which to enumerate
    # RecordSets.
    # @param top [String] Query parameters. If null is passed returns the default
    # number of zones.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<RecordSet>] operation results.
    #
    def list_all_in_resource_group(resource_group_name, zone_name, top = nil, custom_headers = nil)
      first_page = list_all_in_resource_group_as_lazy(resource_group_name, zone_name, top, custom_headers)
      first_page.get_all_items
    end

    #
    # Lists all RecordSets in a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the zone.
    # @param zone_name [String] The name of the zone from which to enumerate
    # RecordSets.
    # @param top [String] Query parameters. If null is passed returns the default
    # number of zones.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_all_in_resource_group_with_http_info(resource_group_name, zone_name, top = nil, custom_headers = nil)
      list_all_in_resource_group_async(resource_group_name, zone_name, top, custom_headers).value!
    end

    #
    # Lists all RecordSets in a DNS zone.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the zone.
    # @param zone_name [String] The name of the zone from which to enumerate
    # RecordSets.
    # @param top [String] Query parameters. If null is passed returns the default
    # number of zones.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_all_in_resource_group_async(resource_group_name, zone_name, top = nil, custom_headers = nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'zone_name is nil' if zone_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/dnszones/{zoneName}/recordsets'
      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'zoneName' => zone_name,'subscriptionId' => @client.subscription_id},
          query_params: {'$top' => top,'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {})
      }

      request_url = @base_url || @client.base_url

      request = MsRest::HttpOperationRequest.new(request_url, path_template, :get, options)
      promise = request.run_promise do |req|
        @client.credentials.sign_request(req) unless @client.credentials.nil?
      end

      promise = promise.then do |http_response|
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(request, http_response, error_model)
        end

        # Create Result
        result = MsRestAzure::AzureOperationResponse.new(request, http_response)
        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = RecordSetListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response, 'result.body')
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists the RecordSets of a specified type in a DNS zone.
    #
    # @param next_page_link [String] The NextLink from the previous successful
    # call to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [RecordSetListResult] operation results.
    #
    def list_by_type_next(next_page_link, custom_headers = nil)
      response = list_by_type_next_async(next_page_link, custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Lists the RecordSets of a specified type in a DNS zone.
    #
    # @param next_page_link [String] The NextLink from the previous successful
    # call to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_type_next_with_http_info(next_page_link, custom_headers = nil)
      list_by_type_next_async(next_page_link, custom_headers).value!
    end

    #
    # Lists the RecordSets of a specified type in a DNS zone.
    #
    # @param next_page_link [String] The NextLink from the previous successful
    # call to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_type_next_async(next_page_link, custom_headers = nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'
      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {})
      }

      request_url = @base_url || @client.base_url

      request = MsRest::HttpOperationRequest.new(request_url, path_template, :get, options)
      promise = request.run_promise do |req|
        @client.credentials.sign_request(req) unless @client.credentials.nil?
      end

      promise = promise.then do |http_response|
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(request, http_response, error_model)
        end

        # Create Result
        result = MsRestAzure::AzureOperationResponse.new(request, http_response)
        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = RecordSetListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response, 'result.body')
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists all RecordSets in a DNS zone.
    #
    # @param next_page_link [String] The NextLink from the previous successful
    # call to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [RecordSetListResult] operation results.
    #
    def list_all_in_resource_group_next(next_page_link, custom_headers = nil)
      response = list_all_in_resource_group_next_async(next_page_link, custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Lists all RecordSets in a DNS zone.
    #
    # @param next_page_link [String] The NextLink from the previous successful
    # call to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_all_in_resource_group_next_with_http_info(next_page_link, custom_headers = nil)
      list_all_in_resource_group_next_async(next_page_link, custom_headers).value!
    end

    #
    # Lists all RecordSets in a DNS zone.
    #
    # @param next_page_link [String] The NextLink from the previous successful
    # call to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_all_in_resource_group_next_async(next_page_link, custom_headers = nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'
      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {})
      }

      request_url = @base_url || @client.base_url

      request = MsRest::HttpOperationRequest.new(request_url, path_template, :get, options)
      promise = request.run_promise do |req|
        @client.credentials.sign_request(req) unless @client.credentials.nil?
      end

      promise = promise.then do |http_response|
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(request, http_response, error_model)
        end

        # Create Result
        result = MsRestAzure::AzureOperationResponse.new(request, http_response)
        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = RecordSetListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response, 'result.body')
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

  end
end
